package org.snowchen.GreatSonServer.programinit;

import java.security.NoSuchAlgorithmException;
import java.sql.*;
import java.util.Scanner;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;

import static org.snowchen.GreatSonServer.programinit.EncryptionModule.hashPassword;

public class UserLogin {
    private Scanner scan = new Scanner(System.in);
    private String user_name;
    private String user_password;

    public UserLogin() {
        while (true) {
            System.out.println("请输入用户名:");
            this.user_name = scan.nextLine();
            if (this.user_name.isEmpty()) {
                System.out.println("用户名不能为空，请重新输入:");
                continue;
            }
            if (!isUsernameExists(this.user_name)) {
                System.out.println("用户名不存在，请重新输入:");
                continue;
            }

            // 检查用户上次登录时间
            LocalDateTime lastLoginTime = getLastLoginTime(this.user_name);
            LocalDateTime currentTime = LocalDateTime.now();
            long minutesDifference = ChronoUnit.MINUTES.between(lastLoginTime, currentTime);

            if (minutesDifference <= 10) {
                System.out.println("已为您自动登录");
                break;
            } else {
                System.out.println("上次登录时间超过10分钟，请输入密码:");
                while (true) {
                    System.out.println("请输入密码:");
                    this.user_password = scan.nextLine();
                    if (this.user_password.isEmpty()) {
                        System.out.println("密码不能为空，请重新输入:");
                        continue;
                    }
                    try {
                        this.user_password = hashPassword(this.user_password);
                        if (verifyPassword(this.user_name, this.user_password)) {
                            System.out.println("登录成功！");
                            break;
                        } else {
                            System.out.println("密码错误，请重新输入密码！");
                        }
                    } catch (NoSuchAlgorithmException e) {
                        System.err.println("加密错误: " + e.getMessage());
                    }
                }
            }
            break;
        }
    }

    /**
     * 检查用户名是否存在
     *
     * @param username 用户名
     * @return 如果用户名存在，则返回true；否则返回false
     */
    private boolean isUsernameExists(String username) {
        String url = "jdbc:sqlite:identifier.sqlite";
        String sql = "SELECT COUNT(*) FROM user_data WHERE user_name = ?";

        try (Connection conn = DriverManager.getConnection(url);
             PreparedStatement pstmt = conn.prepareStatement(sql)) {

            pstmt.setString(1, username);
            try (ResultSet rs = pstmt.executeQuery()) {
                if (rs.next()) {
                    int count = rs.getInt(1);
                    return count > 0;
                }
            }
        } catch (SQLException e) {
            System.err.println("数据库错误: " + e.getMessage());
        }
        return false;
    }

    /**
     * 获取用户的上次登录时间
     *
     * @param username 用户名
     * @return 用户的上次登录时间
     */
    private LocalDateTime getLastLoginTime(String username) {
        String url = "jdbc:sqlite:identifier.sqlite";
        String sql = "SELECT user_last_login_time FROM user_data WHERE user_name = ?";

        try (Connection conn = DriverManager.getConnection(url);
             PreparedStatement pstmt = conn.prepareStatement(sql)) {

            pstmt.setString(1, username);
            try (ResultSet rs = pstmt.executeQuery()) {
                if (rs.next()) {
                    Timestamp timestamp = rs.getTimestamp("user_last_login_time");
                    return timestamp.toLocalDateTime();
                }
            }
        } catch (SQLException e) {
            System.err.println("数据库错误: " + e.getMessage());
        }
        return LocalDateTime.now(); // 如果获取失败，默认为当前时间
    }

    /**
     * 验证密码是否正确
     *
     * @param username 用户名
     * @param password 加密后的密码
     * @return 如果密码正确，则返回true；否则返回false
     */
    private boolean verifyPassword(String username, String password) {
        String url = "jdbc:sqlite:identifier.sqlite";
        String sql = "SELECT user_password FROM user_data WHERE user_name = ?";

        try (Connection conn = DriverManager.getConnection(url);
             PreparedStatement pstmt = conn.prepareStatement(sql)) {

            pstmt.setString(1, username);
            try (ResultSet rs = pstmt.executeQuery()) {
                if (rs.next()) {
                    String storedPassword = rs.getString("user_password");
                    return storedPassword.equals(password);
                }
            }
        } catch (SQLException e) {
            System.err.println("数据库错误: " + e.getMessage());
        }
        return false;
    }

    public static void main(String[] args) {
        new UserLogin();
    }
}


