package org.snowchen.GreatSonServer.programinit;

import java.security.NoSuchAlgorithmException;
import java.sql.*;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Scanner;
import java.util.UUID;

import static org.snowchen.GreatSonServer.programinit.EncryptionModule.hashPassword;

public class UserRegister {
    private Scanner scan = new Scanner(System.in);
    private String user_name;
    private String user_password;
    private String uuid;
    private String user_register_time;
    private String user_last_login_time;
    private int user_permission_level;
    private String[] UserData; // 将UserData数组定义为私有

    public UserRegister() {
        UserData = new String[6];
        // 用户信息初始化
        System.out.println("请输入用户名:");
        this.user_name = scan.nextLine();
        while (this.user_name.isEmpty() || isUsernameExists(this.user_name)) {
            if (this.user_name.isEmpty()) {
                System.out.println("用户名不能为空，请重新输入:");
            } else {
                System.out.println("已经有同名用户，请重新输入用户名:");
            }
            this.user_name = scan.nextLine();
        }
        UserData[0] = this.user_name;
        System.out.println("请输入密码:");
        this.user_password = scan.nextLine();
        while (!isStrongPassword(this.user_password)) {
            System.out.println("密码需为8位以上大小写字母和数字组合，请重新输入:");
            this.user_password = scan.nextLine();
        }
        // 使用SHA-256加密密码
        try {
            this.user_password = hashPassword(this.user_password);
        } catch (NoSuchAlgorithmException e) {
            System.err.println("加密错误: " + e.getMessage());
        }
        UserData[1] = this.user_password;
        this.uuid = UUID.randomUUID().toString(); // 使用标准的 UUID 生成方法
        UserData[2] = this.uuid;
        this.user_permission_level = 0;
        UserData[3] = String.valueOf(this.user_permission_level);
        // 获取年月日时分秒的时间戳
        this.user_register_time = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        UserData[4] = this.user_register_time;
        this.user_last_login_time = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        UserData[5] = this.user_last_login_time;
        System.out.println("用户名:" + this.user_name + "\n密码:" + this.user_password + "\nuuid:" + this.uuid + "\n注册时间:" + this.user_register_time);
        insertUserData(UserData);
    }

    // 提供一个公共方法来获取UserData数组
    public String[] getUserData() {
        return UserData;
    }

    public void insertUserData(String[] UserData) {
        String url = "jdbc:sqlite:identifier.sqlite"; // SQLite数据库URL
        String sql = "INSERT INTO user_data (user_name, user_password, user_uuid, user_permission_level, user_register_time, user_last_login_time) VALUES (?, ?, ?, ?, ?,?)";

        try (Connection conn = DriverManager.getConnection(url);
             PreparedStatement pstmt = conn.prepareStatement(sql)) {

            pstmt.setString(1, UserData[0]);
            pstmt.setString(2, UserData[1]);
            pstmt.setString(3, UserData[2]);
            pstmt.setInt(4, Integer.parseInt(UserData[3])); // 假设user_permission_level是整数
            pstmt.setString(5, UserData[4]); // 假设user_register_time是字符串形式的年月日时分秒时间戳
            pstmt.setString(6, UserData[5]);

            int affectedRows = pstmt.executeUpdate();
            if (affectedRows > 0) {
                System.out.println("用户数据插入成功");
            } else {
                System.out.println("用户数据插入失败");
            }
        } catch (SQLException e) {
            System.err.println("数据库错误: " + e.getMessage());
        } catch (Exception e) {
            System.err.println("错误: " + e.getMessage());
        }
    }

    /**
     * 检测用户名是否存在
     *
     * @param username 待检测的用户名
     * @return 如果用户名存在，则返回true；否则返回false
     */
    public boolean isUsernameExists(String username) {
        String url = "jdbc:sqlite:identifier.sqlite"; // SQLite数据库URL
        String sql = "SELECT COUNT(*) FROM user_data WHERE user_name = ?";

        try (Connection conn = DriverManager.getConnection(url);
             PreparedStatement pstmt = conn.prepareStatement(sql)) {

            pstmt.setString(1, username);
            try (ResultSet rs = pstmt.executeQuery()) {
                if (rs.next()) {
                    int count = rs.getInt(1);
                    return count > 0;
                }
            }
        } catch (SQLException e) {
            System.err.println("数据库错误: " + e.getMessage());
        }
        return false;
    }

    private static final String PWD_REGEX = "^(?=.*[a-z])(?=.*[A-Z])(?=.*\\d).{8,}$";

    /**
     * 检测密码是否为强密码
     * 强密码的定义基于预设的正则表达式PWD_REGEX，该表达式代表了密码的复杂性要求
     *
     * @param password 待检测的密码字符串
     * @return 如果密码符合强密码的要求且不为空字符串，则返回true；否则返回false
     */
    public static boolean isStrongPassword(String password) {
        // 密码强度检测
        if (password == null || password.trim().isEmpty()) {
            return false;
        }
        return password.matches(PWD_REGEX);
    }


}



